#ifndef __LIBENET_H__
#define __LIBENET_H__

#include <IEnet.h>


#ifdef __cplusplus
extern "C" {
#endif

	/*
  DLL export directive
*/
#ifdef _DLL_BUILD_
 #define _DLL_EXPOSURE_  __declspec(dllexport)
#else
 #define _DLL_EXPOSURE_  __declspec(dllimport)
#endif



class CEnet:public IEnet
{
	public:
		/** @defgroup global ENet global functions
    @{ 
*/

/** 
  Initializes ENet globally.  Must be called prior to using any functions in
  ENet.
  @returns 0 on success, < 0 on failure
*/
virtual int _ENET_CALLING_CONVENTION_ Initialize (void);

/** 
  Shuts down ENet globally.  Should be called when a program that has
  initialized ENet exits.
*/
virtual void _ENET_CALLING_CONVENTION_ Deinitialize (void);

/** @} */

/** @defgroup private ENet private implementation functions */

/**
  Returns the wall-time in milliseconds.  Its initial value is unspecified
  unless otherwise set.
  */
virtual enet_uint32 _ENET_CALLING_CONVENTION_ Time_get (void);
/**
  Sets the current wall-time in milliseconds.
  */
virtual void _ENET_CALLING_CONVENTION_ Time_set (enet_uint32);

/** @defgroup socket ENet socket functions
    @{
    @ingroup private
*/
virtual ENetSocket _ENET_CALLING_CONVENTION_ Socket_create (ENetSocketType, const ENetAddress *);
virtual ENetSocket _ENET_CALLING_CONVENTION_ Socket_accept (ENetSocket, ENetAddress *);
virtual int        _ENET_CALLING_CONVENTION_ Socket_connect (ENetSocket, const ENetAddress *);
virtual int        _ENET_CALLING_CONVENTION_ Socket_send (ENetSocket, const ENetAddress *, const ENetBuffer *, size_t);
virtual int        _ENET_CALLING_CONVENTION_ Socket_receive (ENetSocket, ENetAddress *, ENetBuffer *, size_t);
virtual int        _ENET_CALLING_CONVENTION_ Socket_wait (ENetSocket, enet_uint32 *, enet_uint32);
virtual void       _ENET_CALLING_CONVENTION_ Socket_destroy (ENetSocket);

/** @} */

/** @defgroup Address ENet address functions
    @{
*/
/** Attempts to resolve the host named by the parameter hostName and sets
    the host field in the address parameter if successful.
    @param address destination to store resolved address
    @param hostName host name to lookup
    @retval 0 on success
    @retval < 0 on failure
    @returns the address of the given hostName in address on success
*/
virtual int _ENET_CALLING_CONVENTION_ Address_set_host (ENetAddress *address, const char *hostName );

/** Attempts to do a reserve lookup of the host field in the address parameter.
    @param address    address used for reverse lookup
    @param hostName   destination for name, must not be NULL
    @param nameLength maximum length of hostName.
    @returns the null-terminated name of the host in hostName on success
    @retval 0 on success
    @retval < 0 on failure
*/
virtual int _ENET_CALLING_CONVENTION_ Address_get_host (const ENetAddress *address, char *hostName, size_t nameLength );

/** @} */

virtual ENetPacket * _ENET_CALLING_CONVENTION_ Packet_create (const void *dataContents, size_t dataLength, enet_uint32 flags);
virtual void    _ENET_CALLING_CONVENTION_  Packet_destroy (ENetPacket *packet );
virtual int     _ENET_CALLING_CONVENTION_  Packet_resize  (ENetPacket *packet, size_t dataLength );

virtual ENetHost * _ENET_CALLING_CONVENTION_ Host_create (const ENetAddress *address, size_t peerCount, enet_uint32 incomingBandwidth, enet_uint32 outgoingBandwidth );
virtual void       _ENET_CALLING_CONVENTION_ Host_destroy (ENetHost *host );
virtual ENetPeer * _ENET_CALLING_CONVENTION_ Host_connect (ENetHost *host, const ENetAddress *address, size_t channelCount );
virtual int        _ENET_CALLING_CONVENTION_ Host_service (ENetHost *, ENetEvent *, enet_uint32);
virtual void       _ENET_CALLING_CONVENTION_ Host_flush (ENetHost *);
virtual void       _ENET_CALLING_CONVENTION_ Host_broadcast (ENetHost *, enet_uint8, ENetPacket *);
virtual void       _ENET_CALLING_CONVENTION_ Host_bandwidth_limit (ENetHost *, enet_uint32, enet_uint32);
virtual void       _ENET_CALLING_CONVENTION_ Host_bandwidth_throttle (ENetHost *);

virtual int         _ENET_CALLING_CONVENTION_ Peer_send (ENetPeer *, enet_uint8, ENetPacket *);
virtual ENetPacket* _ENET_CALLING_CONVENTION_ Peer_receive (ENetPeer *, enet_uint8);
virtual void        _ENET_CALLING_CONVENTION_ Peer_ping (ENetPeer *);
virtual void        _ENET_CALLING_CONVENTION_ Peer_reset (ENetPeer *);
virtual void        _ENET_CALLING_CONVENTION_ Peer_disconnect (ENetPeer *);
virtual void        _ENET_CALLING_CONVENTION_ Peer_disconnect_now (ENetPeer *);
virtual void        _ENET_CALLING_CONVENTION_ Peer_throttle_configure (ENetPeer *, enet_uint32, enet_uint32, enet_uint32);
virtual int         _ENET_CALLING_CONVENTION_ Peer_throttle (ENetPeer *, enet_uint32);
virtual void        _ENET_CALLING_CONVENTION_ Peer_reset_queues (ENetPeer *);
virtual ENetOutgoingCommand * _ENET_CALLING_CONVENTION_ Peer_queue_outgoing_command (ENetPeer *, const ENetProtocol *, ENetPacket *, enet_uint32, enet_uint16);
virtual ENetIncomingCommand * _ENET_CALLING_CONVENTION_ Peer_queue_incoming_command (ENetPeer *, const ENetProtocol *, ENetPacket *, enet_uint32);
virtual ENetAcknowledgement * _ENET_CALLING_CONVENTION_ Peer_queue_acknowledgement (ENetPeer *, const ENetProtocol *, enet_uint32);

};

///Class factory functions
_DLL_EXPOSURE_ int _ENET_CALLING_CONVENTION_ GetEnetInstance(IEnet**);


#ifdef __cplusplus
}
#endif
#endif /* __LIBENET_H__ */
